#include "Arduino.h"
#include "TeensyStep.h"
//#include "ADNS2610.h"
#include <Encoder.h>
#include <SD.h>
#include <SPI.h>

int i = 0;

const int chipSelect = BUILTIN_SDCARD;

//Encoder variables
#define  A_PHASE 14 // green cable
#define  B_PHASE 2 // white cable

const int gear_ratio = 6;

//Encoder object
Encoder encoder(A_PHASE, B_PHASE);

float ppr = gear_ratio*2458; //2458 is the number of pulses per encoder revolution
float deg_pulse = 360/ppr;
float encoder_angle = 0;

//Stepper configuration
const int steps_rev = 25000*gear_ratio;//6 times 4000 steps/revolution

//Maximum angle values 
int max_angle_deg = 90;
int min_angle_deg = 0; 
int max_angle_step = (max_angle_deg*steps_rev)/360;
int min_angle_step = (min_angle_deg*steps_rev)/360;

int angle_array[] = {45, 90, 135, 180, 225, 270, 315, 360, 0, 0, 0};

const int omega_deg = 20; // Angular velocity in deg*s^-1
const int alpha_deg = 1200; // Angular acceleration in deg*s^-2

const int omega_steps = (omega_deg*steps_rev)/360; // Angular velocity in steps*s^-1
const int alpha_steps = (alpha_deg*steps_rev)/360; // Angular acceleration in steps*s^-2

float  desired_position = 0;
float  actual_position = 0;

constexpr int stpPin = 5, dirPin = 6;
Stepper stepper(stpPin, dirPin);
StepControl controller;

int32_t Setpoint = 0;
bool state = 0;

int hallSensorPin = 4;

int DAC_out_0;
int DAC_out_1;


void Homing()
{
  while (digitalRead(hallSensorPin) == HIGH)
  {
    TurnCounterClockwise();
  }
  delay(1000);
}

void TurnCounterClockwise ()
{
   digitalWrite(6,1);
   digitalWrite(5,HIGH);
   delayMicroseconds(100);
   digitalWrite(5,LOW);
   delayMicroseconds(100);
}

void setup()
{

  Serial.print("Initializing SD card...");
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect))
  {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
  

  pinMode(hallSensorPin, INPUT);
  
  analogWriteResolution(12);
  
  Serial.begin(112500);

  Homing();
  
  stepper
      .setMaxSpeed(omega_steps) 
      .setAcceleration(alpha_steps); 

    //Encoder variable to zero
  encoder.write(0);

}

elapsedMillis outputTimer = 0;
//Timer between setpoints
elapsedMillis reSetpointTimer = 0;
//Timer for the Pulse before starting the rotation
elapsedMillis PulseTimer = 0;

void loop()
{

  stepper.setTargetAbs(Setpoint);
  controller.moveAsync(stepper);
  
  while(controller.isRunning())
  {              
    // wait until the movement is finished
    encoder_angle = float(encoder.read()*deg_pulse);
    actual_position = stepper.getPosition()*0.0024; //0.015=360deg/150000step
    
    // open the file.
    File dataFile = SD.open("201200.txt", FILE_WRITE);
  
    // if the file is available, write to it:
    if (dataFile) 
    {
        dataFile.print(encoder_angle);
        dataFile.print(";");
        dataFile.print(actual_position);
        dataFile.print(";");
        dataFile.print(Setpoint);
        dataFile.print(";");
        dataFile.println(millis());
        dataFile.close();
        
         Serial.print(encoder_angle);
        // Serial.print(encoder.read());
        Serial.print("    ");
        Serial.print(actual_position);
        Serial.print("    ");
        Serial.print(angle_array[i]);
        Serial.print("    ");
        Serial.println(millis()); 
      }  
    // if the file isn't open, pop up an error:
    else 
      {
        Serial.println("error opening datalog.txt");
      }               
  }            


  if(reSetpointTimer > 25000)  // new target every 20 seconds
  {
    reSetpointTimer = 0;

    //Change the state variable
    state = !state;
    
    if (state == 0)
    {
      //Setpoint = max_angle_step;
      
      //Setpoint = random(0,360)*150000/360; // Random number between 0 and 360 
      
      Setpoint = angle_array[i]*150000/360; // It will go to all positions of the array
      i= i+1;
    }
    if (state == 1)
    {
      Setpoint = min_angle_step;
    }
  }
//  Serial.print(encoder_angle);
//  // Serial.print(encoder.read());
//  Serial.print("    ");
//  Serial.print(actual_position);
//  Serial.print("    ");
//  Serial.print(angle_array[i]);
//  Serial.print("    ");
//  Serial.println(millis()); 
      
    // open the file.
  File dataFile = SD.open("201200.txt", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) 
  {
      dataFile.print(encoder_angle);
      dataFile.print(";");
      dataFile.print(actual_position);
      dataFile.print(";");
      dataFile.print(angle_array[i]);
      dataFile.print(";");
      dataFile.println(millis());
      dataFile.close();
      
      Serial.print(encoder_angle);
      // Serial.print(encoder.read());
      Serial.print("    ");
      Serial.print(actual_position);
      Serial.print("    ");
      Serial.print(angle_array[i]);
      Serial.print("    ");
      Serial.println(millis()); 
    }  
  // if the file isn't open, pop up an error:
  else 
    {
      Serial.println("error opening datalog.txt");
    } 

  //analogWrite(A22,int(float(stepper.getPosition())*0.015));
  
 
    //DAC_out_1 = int(5.6875*1.9*(actual_position) );
    //DAC_out_0 = int(5.6875*desired_position );

}
